import "package:flutter/material.dart";
import 'package:flutter_colorpicker/flutter_colorpicker.dart';
import 'package:realtime_whiteboard/model/custom_painter.dart';
import 'package:realtime_whiteboard/model/touch_points.dart';
import 'package:socket_io_client/socket_io_client.dart' as IO;

class PaintScreen extends StatefulWidget {
  const PaintScreen({Key key}) : super(key: key);

  @override
  _PaintScreenState createState() => _PaintScreenState();
}

class _PaintScreenState extends State<PaintScreen> {
  GlobalKey globalKey = GlobalKey();
  IO.Socket socket;
  List<TouchPoints> points = [];
  double opacity = 1.0;
  StrokeCap strokeType = StrokeCap.round;
  Color selectedColor;
  double strokeWidth;

  @override
  void initState() {
    super.initState();
    connectToSocket();
    selectedColor = Colors.black;
    strokeWidth = 2.0;
  }

  void connectToSocket() {
    socket = IO.io(
        "https://historic-joshua-tree-83314.herokuapp.com", <String, dynamic>{
      "transports": ["websocket"],
      "autoConnect": false,
    });
    socket.connect();

    socket.onConnect((data) {
      // adding points to the screen
      socket.on("points", (point) {
        if (point["details"] != null) {
          setState(() {
            points.add(
              TouchPoints(
                  points: Offset((point["details"]["dx"]).toDouble(),
                      (point["details"]["dy"]).toDouble()),
                  paint: Paint()
                    ..strokeCap = strokeType
                    ..isAntiAlias = true
                    ..color = selectedColor.withOpacity(opacity)
                    ..strokeWidth = strokeWidth),
            );
          });
        } else {
          setState(() {
            points.add(null);
          });
        }
      });

      // changing stroke width of pen
      socket.on(
          "stroke-width",
          (stroke) => this.setState(() {
                strokeWidth = stroke.toDouble();
              }));

      // changing the color of pen
      socket.on("color-change", (colorString) {
        int value = int.parse(colorString, radix: 16);
        Color otherColor = new Color(value);
        setState(() {
          selectedColor = otherColor;
        });
      });

      // clearing off the screen with clean button
      socket.on(
          "clear-screen",
          (data) => this.setState(() {
                points.clear();
              }));
    });
  }

  @override
  void dispose() {
    socket.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final double width = MediaQuery.of(context).size.width;
    final double height = MediaQuery.of(context).size.height;

    void selectColor() {
      showDialog(
        context: context,
        builder: (context) => AlertDialog(
          title: const Text('Color Chooser'),
          content: SingleChildScrollView(
            child: BlockPicker(
              pickerColor: selectedColor,
              onColorChanged: (color) {
                String colorString = color.toString();
                String valueString = colorString.split('(0x')[1].split(')')[0];
                Map map = {
                  "color": valueString,
                };
                socket.emit("color-change", map);
              },
            ),
          ),
          actions: <Widget>[
            TextButton(
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: Text("Close"))
          ],
        ),
      );
    }

    return Scaffold(
      backgroundColor: Colors.white,
      body: Stack(
        children: [
          Column(
            children: [
              Container(
                width: width,
                height: height * 0.9,
                child: GestureDetector(
                  onPanUpdate: (details) {
                    socket.emit("paint", {
                      "details": {
                        "dx": details.localPosition.dx,
                        "dy": details.localPosition.dy
                      },
                    });
                  },
                  onPanStart: (details) {
                    socket.emit("paint", {
                      "details": {
                        "dx": details.localPosition.dx,
                        "dy": details.localPosition.dy
                      },
                    });
                  },
                  onPanEnd: (details) {
                    socket.emit("paint", {
                      "details": null,
                    });
                  },
                  child: SizedBox.expand(
                    child: ClipRRect(
                      borderRadius: BorderRadius.all(Radius.circular(20.0)),
                      child: RepaintBoundary(
                        key: globalKey,
                        child: CustomPaint(
                          size: Size.infinite,
                          painter: MyCustomPainter(pointsList: points),
                        ),
                      ),
                    ),
                  ),
                ),
              ),
              Row(
                children: <Widget>[
                  IconButton(
                      icon: Icon(
                        Icons.color_lens,
                        color: selectedColor,
                      ),
                      onPressed: () {
                        selectColor();
                      }),
                  Expanded(
                    child: Slider(
                      min: 1.0,
                      max: 10.0,
                      label: "Stroke $strokeWidth",
                      activeColor: selectedColor,
                      value: strokeWidth,
                      onChanged: (double value) {
                        socket.emit("stroke-width", value);
                      },
                    ),
                  ),
                  IconButton(
                      icon: Icon(
                        Icons.layers_clear,
                        color: Colors.black,
                      ),
                      onPressed: () {
                        socket.emit("clean-screen");
                      }),
                ],
              )
            ],
          ),
        ],
      ),
    );
  }
}
